@@ -87,7 +87,14 @@ public final class WriterBasedJsonGenerator
     public Object getOutputTarget() {
         return _writer;
     }
-    
+
+    @Override
+    public int getOutputBuffered() {
+        // Assuming tail and head are kept but... trust and verify:
+        int len = _outputTail - _outputHead;
+        return Math.max(0, len);
+    }
+
     /*
     /**********************************************************
     /* Overridden methods
@@ -319,7 +326,7 @@ public final class WriterBasedJsonGenerator
     @Override
     public void writeString(String text) throws IOException
     {
-        _verifyValueWrite("write text value");
+        _verifyValueWrite(WRITE_STRING);
         if (text == null) {
             _writeNull();
             return;
@@ -339,7 +346,7 @@ public final class WriterBasedJsonGenerator
     @Override
     public void writeString(char[] text, int offset, int len) throws IOException
     {
-        _verifyValueWrite("write text value");
+        _verifyValueWrite(WRITE_STRING);
         if (_outputTail >= _outputEnd) {
             _flushBuffer();
         }
@@ -355,7 +362,7 @@ public final class WriterBasedJsonGenerator
     @Override
     public void writeString(SerializableString sstr) throws IOException
     {
-        _verifyValueWrite("write text value");
+        _verifyValueWrite(WRITE_STRING);
         if (_outputTail >= _outputEnd) {
             _flushBuffer();
         }
@@ -507,7 +514,7 @@ public final class WriterBasedJsonGenerator
     public void writeBinary(Base64Variant b64variant, byte[] data, int offset, int len)
         throws IOException, JsonGenerationException
     {
-        _verifyValueWrite("write binary value");
+        _verifyValueWrite(WRITE_BINARY);
         // Starting quotes
         if (_outputTail >= _outputEnd) {
             _flushBuffer();
@@ -526,7 +533,7 @@ public final class WriterBasedJsonGenerator
             InputStream data, int dataLength)
         throws IOException, JsonGenerationException
     {
-        _verifyValueWrite("write binary value");
+        _verifyValueWrite(WRITE_BINARY);
         // Starting quotes
         if (_outputTail >= _outputEnd) {
             _flushBuffer();
@@ -564,7 +571,7 @@ public final class WriterBasedJsonGenerator
     @Override
     public void writeNumber(short s) throws IOException
     {
-        _verifyValueWrite("write number");
+        _verifyValueWrite(WRITE_NUMBER);
         if (_cfgNumbersAsStrings) {
             _writeQuotedShort(s);
             return;
@@ -588,7 +595,7 @@ public final class WriterBasedJsonGenerator
     @Override
     public void writeNumber(int i) throws IOException
     {
-        _verifyValueWrite("write number");
+        _verifyValueWrite(WRITE_NUMBER);
         if (_cfgNumbersAsStrings) {
             _writeQuotedInt(i);
             return;
@@ -612,7 +619,7 @@ public final class WriterBasedJsonGenerator
     @Override
     public void writeNumber(long l) throws IOException
     {
-        _verifyValueWrite("write number");
+        _verifyValueWrite(WRITE_NUMBER);
         if (_cfgNumbersAsStrings) {
             _writeQuotedLong(l);
             return;
@@ -638,11 +645,11 @@ public final class WriterBasedJsonGenerator
     @Override
     public void writeNumber(BigInteger value) throws IOException
     {
-        _verifyValueWrite("write number");
+        _verifyValueWrite(WRITE_NUMBER);
         if (value == null) {
             _writeNull();
         } else if (_cfgNumbersAsStrings) {
-            _writeQuotedRaw(value);
+            _writeQuotedRaw(value.toString());
         } else {
             writeRaw(value.toString());
         }
@@ -659,7 +666,7 @@ public final class WriterBasedJsonGenerator
             return;
         }
         // What is the max length for doubles? 40 chars?
-        _verifyValueWrite("write number");
+        _verifyValueWrite(WRITE_NUMBER);
         writeRaw(String.valueOf(d));
     }
 
@@ -673,7 +680,7 @@ public final class WriterBasedJsonGenerator
             return;
         }
         // What is the max length for floats?
-        _verifyValueWrite("write number");
+        _verifyValueWrite(WRITE_NUMBER);
         writeRaw(String.valueOf(f));
     }
 
@@ -681,11 +688,12 @@ public final class WriterBasedJsonGenerator
     public void writeNumber(BigDecimal value) throws IOException
     {
         // Don't really know max length for big decimal, no point checking
-        _verifyValueWrite("write number");
+        _verifyValueWrite(WRITE_NUMBER);
         if (value == null) {
             _writeNull();
         } else if (_cfgNumbersAsStrings) {
-            _writeQuotedRaw(value);
+            String raw = isEnabled(Feature.WRITE_BIGDECIMAL_AS_PLAIN) ? value.toPlainString() : value.toString();
+            _writeQuotedRaw(raw);
         } else if (isEnabled(Feature.WRITE_BIGDECIMAL_AS_PLAIN)) {
             writeRaw(value.toPlainString());
         } else {
@@ -696,7 +704,7 @@ public final class WriterBasedJsonGenerator
     @Override
     public void writeNumber(String encodedValue) throws IOException
     {
-        _verifyValueWrite("write number");
+        _verifyValueWrite(WRITE_NUMBER);
         if (_cfgNumbersAsStrings) {
             _writeQuotedRaw(encodedValue);            
         } else {
@@ -704,13 +712,13 @@ public final class WriterBasedJsonGenerator
         }
     }
 
-    private void _writeQuotedRaw(Object value) throws IOException
+    private void _writeQuotedRaw(String value) throws IOException
     {
         if (_outputTail >= _outputEnd) {
             _flushBuffer();
         }
         _outputBuffer[_outputTail++] = '"';
-        writeRaw(value.toString());
+        writeRaw(value);
         if (_outputTail >= _outputEnd) {
             _flushBuffer();
         }
@@ -720,7 +728,7 @@ public final class WriterBasedJsonGenerator
     @Override
     public void writeBoolean(boolean state) throws IOException
     {
-        _verifyValueWrite("write boolean value");
+        _verifyValueWrite(WRITE_BOOLEAN);
         if ((_outputTail + 5) >= _outputEnd) {
             _flushBuffer();
         }
@@ -743,7 +751,7 @@ public final class WriterBasedJsonGenerator
 
     @Override
     public void writeNull() throws IOException {
-        _verifyValueWrite("write null value");
+        _verifyValueWrite(WRITE_NULL);
         _writeNull();
     }
 
@@ -839,8 +847,7 @@ public final class WriterBasedJsonGenerator
     }
 
     @Override
-    public void close()
-        throws IOException
+    public void close() throws IOException
     {
         super.close();
 
@@ -862,6 +869,8 @@ public final class WriterBasedJsonGenerator
             }
         }
         _flushBuffer();
+        _outputHead = 0;
+        _outputTail = 0;
 
         /* 25-Nov-2008, tatus: As per [JACKSON-16] we are not to call close()
          *   on the underlying Reader, unless we "own" it, or auto-closing
